/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2007                               **
**                                                                        **
**                    MODULE LPC2378 UART0 Service                        **
**                                                                        **
**                     Mikrokontroler ARM: LPC2378                        **
**                                                                        **
**                     Wersja z dnia : 2007.03.25                         **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/

#include <LPC23xx.h>
#include "uart0.h"
#include "types.h"
#include "system.h"
#include "irqvectors.h"
#include "serialconst.h"

#define DefaultLEDDelay       44

#define UART0InterruptMask ( 1 << UART0_Interrupt )

#define InpSerialBuffSize     64
#define OutSerialBuffSize     128
                              /* 64 */

typedef struct {
                   USHORT     RdDataPointer ;
                   USHORT     WrDataPointer ;
                   UCHAR      Buffer [ InpSerialBuffSize ] ;
                 } InpSerialBufferRecT ;

typedef struct {
                   USHORT     RdDataPointer ;
                   USHORT     WrDataPointer ;
                   USHORT     BusyStatus ;
                   UCHAR      Buffer [ OutSerialBuffSize ] ;
                 } OutSerialBufferRecT ;


static volatile InpSerialBufferRecT UART0InpBuffer ;
static volatile OutSerialBufferRecT UART0OutBuffer ;


__irq void UART0IrqHandler ( void )
{
  UCHAR Data ;
  UCHAR U0IrqStatus ;
  /*-------------------------------------------------------------------------*/
  for ( ; ; )
  {
    U0IrqStatus = ( UCHAR ) U0IIR ;
    if ( U0IrqStatus & 0x01 )
    {
      VICVectAddr = 0 ;
      return ;
    } /* if */ ;
    switch ( ( U0IrqStatus >> 1 ) & 0x03 )
    {
      case 1             :   /* 0 1 - transmitter */
        if ( UART0OutBuffer . RdDataPointer == UART0OutBuffer . WrDataPointer )
        {
          UART0OutBuffer . BusyStatus = 0 ;
          break ;
        } /* if */ ;
        Data = UART0OutBuffer . Buffer [ UART0OutBuffer . RdDataPointer ] ;
        UART0OutBuffer . RdDataPointer ++ ;
        if ( UART0OutBuffer . RdDataPointer >= OutSerialBuffSize )
          UART0OutBuffer . RdDataPointer = 0 ;
        U0THR = Data ;
        break ;
      case 2             :   /* 1 0 - reciever data */
        Data = U0RBR ;
        UART0InpBuffer . Buffer [ UART0InpBuffer . WrDataPointer ] = Data ;
        UART0InpBuffer . WrDataPointer ++ ;
        if ( UART0InpBuffer . WrDataPointer >= InpSerialBuffSize )
          UART0InpBuffer . WrDataPointer = 0 ;
        break ;
      default            :
        VICVectAddr = 0 ;
        return ;
    } /* switch */ ;
  } /* for */ ;
} /* UART0IrqHandler */


void UART0InputReset ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0InpBuffer . RdDataPointer = 0 ;
  UART0InpBuffer . WrDataPointer = 0 ;
} /* UART0InputReset */


static void UART0OutputReset ( void )
{
  /*-------------------------------------------------------------------------*/
  UART0OutBuffer . RdDataPointer = 0 ;
  UART0OutBuffer . WrDataPointer = 0 ;
  UART0OutBuffer . BusyStatus = 0 ;
} /* UART0OutputReset */


static void UART0DisableInterrupt ( void )
{
  /*-------------------------------------------------------------------------*/
  VICIntEnClr = UART0InterruptMask ;
} /* UART0DisableInterrupt */


static void UART0EnableInterrupt ( void )
{
  /*-------------------------------------------------------------------------*/
  VICIntEnable = UART0InterruptMask ;
} /* UART0EnableInterrupt */


void UART0Send ( UCHAR Data )
{
  USHORT WStop ;
  /*-------------------------------------------------------------------------*/
  UART0DisableInterrupt ( ) ;
  if ( ! UART0OutBuffer . BusyStatus )
  {
    U0THR = Data ;
    UART0OutBuffer . BusyStatus = 1 ;
    UART0EnableInterrupt ( ) ;
  } /* if ... */
  else
  {
    for ( ; ; )
    {
      WStop = UART0OutBuffer . WrDataPointer + 1 ;
      if ( WStop >= OutSerialBuffSize )
        WStop = 0 ;
      if ( WStop != UART0OutBuffer . RdDataPointer )
      {
        UART0OutBuffer . Buffer [ UART0OutBuffer . WrDataPointer ] = Data ;
        UART0OutBuffer . WrDataPointer = WStop ;
        UART0EnableInterrupt ( ) ;
        break ;
      } /* if ... */
      UART0EnableInterrupt ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      NOP ( ) ;
      UART0DisableInterrupt ( ) ;
    } /* for */ ;
  } /* if ... else */ ;
} /* UART0Send */


void UART0SendString ( UCHAR * String )
{
  /*-------------------------------------------------------------------------*/
  while ( * String )
  {
    UART0Send ( * String ) ;
    String ++ ;
  } /* while */ ;
} /* UART0SendString */


ULONG UART0DataPresent ( void )
{
  ULONG Result ;
  /*-------------------------------------------------------------------------*/
  Result = FALSE ;
  UART0DisableInterrupt ( ) ;
  if ( UART0InpBuffer . RdDataPointer != UART0InpBuffer . WrDataPointer )
  {
    Result = TRUE ;
  } /* if */ ;
  UART0EnableInterrupt ( ) ;
  return ( Result ) ;
} /* UART0DataPresent */


UCHAR UART0GetData ( void )
{
  UCHAR Data ;
  /*-------------------------------------------------------------------------*/
  UART0DisableInterrupt ( ) ;
  if ( UART0InpBuffer . RdDataPointer == UART0InpBuffer . WrDataPointer )
  {
    UART0EnableInterrupt ( ) ;
    return ( 0 ) ;
  } /* if */ ;
  Data = UART0InpBuffer . Buffer [ UART0InpBuffer . RdDataPointer ] ;
  UART0InpBuffer . RdDataPointer ++ ;
  if ( UART0InpBuffer . RdDataPointer >= InpSerialBuffSize )
    UART0InpBuffer . RdDataPointer = 0 ;
  UART0EnableInterrupt ( ) ;
  return ( Data ) ;
} /* UARTGetData */


UCHAR UART0Working ( void )
{
  UCHAR Data ;
  /*-------------------------------------------------------------------------*/
  if ( UART0OutBuffer . BusyStatus )
    return FALSE ;
  Data = U0LSR ;
  if ( Data  & 0x40 )
    return ( TRUE ) ;
  else
    return ( FALSE ) ;
  } /* UART0Working */


void UART0HardwInit( ULONG PeriphClk ,
                     UCHAR SerialMode ,
                     ULONG SerialSpeed )
{
  ULONG Fdiv ;
  ULONG Fmod ;
  ULONG LDivider ;
  ULONG HDivider ;
  /*-------------------------------------------------------------------------*/
  UART0DisableInterrupt ( ) ;
  UART0InputReset ( ) ;
  UART0OutputReset ( ) ;
  Fdiv = ( PeriphClk / 16 ) / SerialSpeed ;
  Fmod = ( PeriphClk / 16 ) % SerialSpeed ;
  if ( ( Fmod << 1 ) >= SerialSpeed )
    Fdiv ++ ;
  LDivider = Fdiv % 256 ;
  HDivider = Fdiv / 256 ;
  U0FDR = 0x10 ;
  U0LCR = 0x80 ;
  U0DLL = LDivider ;
  U0DLM = HDivider ;
  U0LCR = SerialMode ;
  U0FCR = 0x07 ;
  VICVectAddr6  = ( unsigned long ) UART0IrqHandler ;
  VICVectCntl6  = 15 ;
  UART0EnableInterrupt ( ) ;
  U0IER = RecvDataInterruptEnable | TransmDataInterruptEnable ;
} /* UART0HardwInit */


void UART0InitEnvir ( void )
{
  /*-------------------------------------------------------------------------*/
  PINSEL0 &= ~ 0x000000F0 ;
  PINSEL0 |= 0x00000050 ;
  UART0InputReset ( ) ;
  UART0OutputReset ( ) ;
} /* UART0InitEnvir */


